unit Conector;

interface

uses ResultadoRegistro, Soap.SOAPHTTPClient, Timbrado, Soap.Rio, Comprobante,
  ResultadoTimbre, ResultadoConsulta, ResultadoCancelacion;

type
   TConector = Class
     private
       _usuarioIntegrador     : String;    // The string passed to the constructor
       X: THTTPRIO;
          Interf: TimbradoSoap;
        RIOAccess: IRIOAccess;
       //private TimbradoSoapClient _cliente;
        function ExtraeData(propiedad,valor: string):string;

     // These methods and properties are all usable by instances of the class
     public

       constructor Create(url: string);

       procedure EstableceCredenciales(usuarioIntegrador: String);
       function RegistraEmisor(rfcEmisor: String; rutaCer: String; rutaKey: String; contrasena: String) : TResultadoRegistro;
       function TimbraCFDI(cfdi: TComprobante): TResultadoTimbre overload;
       function TimbraCFDI(strFile: string): TResultadoTimbre overload;
       function ObtieneCFDI(rfcEmisor, folio: String): TResultadoConsulta;
       function CancelaCFDI(rfcEmisor, folio: String): TResultadoCancelacion;
   end;

implementation

uses Util, System.SysUtils, System.Classes, System.TypInfo,
  ComprobanteEmisorRegimenFiscal, ComprobanteConcepto,
  ComprobanteImpuestosTraslado, StrUtils, Soap.EncdDecd,
  System.RegularExpressions;

constructor TConector.Create(url: string);
 begin
   Interf := GetTimbradoSoap(True, url + '?wsdl');
   RIOAccess := Interf as IRIOAccess;
   X := RIOAccess.RIO as THTTPRIO;

   X.WSDLLocation := url + '?wsdl';
   X.Service := 'Timbrado';
   X.Port := 'TimbradoSoap';

 end;

procedure TConector.EstableceCredenciales(usuarioIntegrador: String);
begin
 _usuarioIntegrador:=usuarioIntegrador;
end;


function TConector.RegistraEmisor(rfcEmisor, rutaCer, rutaKey,
  contrasena: String): TResultadoRegistro;

var
  registro: TResultadoRegistro ;
  str: AnsiString;
  str2: AnsiString;
begin

    try
        str := EncodeFile(rutaCer);
        str2 := EncodeFile(rutaKey);

        registro := TResultadoRegistro.Create((X as TimbradoSoap).RegistraEmisor(_usuarioIntegrador, rfcEmisor, str, str2, contrasena));

        //registro := TResultadoRegistro.Create(Timbrado.RegistraEmisor(_usuarioIntegrador, rfcEmisor, str, str2, contrasena));

    except on E: Exception do
      raise Exception.Create('Ocurri un error al registrar al emisor: ' + E.Message);
    end;

    result:= registro;

end;


function TConector.ObtieneCFDI(rfcEmisor, folio: String): TResultadoConsulta;

var
  registro: TResultadoConsulta ;
  str: AnsiString;
  str2: AnsiString;
begin

    try
        registro := TResultadoConsulta.Create((X as TimbradoSoap).ObtieneCFDI (_usuarioIntegrador, rfcEmisor, folio));

    except on E: Exception do
      raise Exception.Create('Ocurri un error al consultar el comprobante: ' + E.Message);
    end;

    result:= registro;

end;


function TConector.CancelaCFDI(rfcEmisor, folio: String): TResultadoCancelacion;

var
  registro: TResultadoCancelacion ;
  str: AnsiString;
  str2: AnsiString;
begin

    try
        registro := TResultadoCancelacion.Create((X as TimbradoSoap).CancelaCFDI (_usuarioIntegrador, rfcEmisor, folio));

    except on E: Exception do
      raise Exception.Create('Ocurri un error al cancelar el comprobante: ' + E.Message);
    end;

    result:= registro;

end;

function TConector.TimbraCFDI(strFile: string): TResultadoTimbre;
var
  timbre: TResultadoTimbre;
  strList: TStringList;
  str: String;
  regimenFiscal: TComprobanteEmisorRegimenFiscal;
  concepto: TComprobanteConcepto;
  //traslado: TComprobanteImpuestosTraslado;
  idComprobante: String;
  emisorRFC: string;
  serie: string;
  folio: string;
   regexpr : TRegEx;
   match   : TMatch;
begin
   strList:=TStringList.Create;
   strList.LoadFromFile(strFile);

   regexpr := TRegEx.Create('cfdi:Emisor rfc="(\S+)"');
   match := regexpr.Match(strList.Text);
   if match.Success then emisorRFC:=match.Groups[1].Value;

   serie:='F';

   regexpr := TRegEx.Create('folio="(\S+)"');
   match := regexpr.Match(strList.Text);
   if match.Success then folio:=match.Groups[1].Value;


   idComprobante := emisorRFC + serie + folio + inttostr(Random($270));
        if (Length(idComprobante)>$19) then idComprobante := leftstr(idComprobante,$19);

        try

            timbre := TResultadoTimbre.Create((X as TimbradoSoap).TimbraCFDI(_usuarioIntegrador,EncodeString(strList.Text),idComprobante));
            strList.Free;
        except on E: Exception do
          raise Exception.Create('Ocurri un error al timbrar: ' + E.Message);
        end;

        result:= timbre;
end;


function TConector.ExtraeData(propiedad,valor: string): string;
begin
       if (valor<>'') then begin
           ExtraeData:= propiedad + '="' + valor + '" ';
       end
       else
       ExtraeData:='';

end;

function TConector.TimbraCFDI(cfdi: TComprobante): TResultadoTimbre;
var
  timbre: TResultadoTimbre;
  strList: TStringList; 
  //str: String;
  regimenFiscal: TComprobanteEmisorRegimenFiscal;
  concepto: TComprobanteConcepto;
  traslado: TComprobanteImpuestosTraslado;
  idComprobante: String;
begin

   strList:=TStringList.Create;

   strList.Add('<?xml version="1.0" encoding="utf-8"?>');
   strList.Add('<cfdi:Comprobante xmlns:cfdi="http://www.sat.gob.mx/cfd/3"' +
          ' xmlns:tfd="http://www.sat.gob.mx/TimbreFiscalDigital" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'+
          ' xsi:schemaLocation="http://www.sat.gob.mx/cfd/3 http://www.sat.gob.mx/sitio_internet/cfd/3/cfdv32.xsd"'+
          ' version="3.2" folio="'+cfdi.folio+
          '" fecha="'+FormatDateTime('yyyy-mm-dd''T''hh:nn:ss',cfdi.fecha)+ //2013-08-25T11:57:00
          '" formaDePago="'+cfdi.formaDePago+
          '" subTotal="'+ floattostr(cfdi.subTotal)+
          '" Moneda="'+  cfdi.moneda+
          '" total="'+ floattostr(cfdi.total)+
          '" tipoDeComprobante="'+GetEnumName(TypeInfo(TComprobanteTipoDeComprobante),integer(cfdi.tipoDeComprobante))+
          '" metodoDePago="'+cfdi.metodoDePago+'"' +
          ' LugarExpedicion="'+cfdi.lugarExpedicion+'">');



        strList.Add('  <cfdi:Emisor rfc="'+cfdi.emisor.rfc+'" nombre="'+cfdi.emisor.nombre+'">');
        strList.Add('    <cfdi:DomicilioFiscal calle="'+cfdi.emisor.domicilioFiscal.calle+
        //'" noExterior="'+cfdi.emisor.domicilioFiscal.noExterior+
        '" ' + ExtraeData('noExterior',cfdi.emisor.domicilioFiscal.noExterior) +
        'colonia="'+cfdi.emisor.domicilioFiscal.colonia+
        '" localidad="'+cfdi.emisor.domicilioFiscal.localidad+
        '" municipio="'+cfdi.emisor.domicilioFiscal.municipio+
        '" estado="'+cfdi.emisor.domicilioFiscal.estado+
        '" pais="'+cfdi.emisor.domicilioFiscal.pais+
        '" codigoPostal="'+cfdi.emisor.domicilioFiscal.codigoPostal+'" />');

         for regimenFiscal in cfdi.emisor.regimenFiscal do
          begin
            strList.Add('    <cfdi:RegimenFiscal Regimen="'+regimenFiscal.regimen+'" />');
          end;

        strList.Add('  </cfdi:Emisor>');


        strList.Add('  <cfdi:Receptor rfc="'+cfdi.receptor.rfc+'" nombre="'+cfdi.receptor.nombre+'">');

        strList.Add('    <cfdi:Domicilio calle="'+cfdi.receptor.domicilio.calle+
        '" noExterior="'+cfdi.receptor.domicilio.noExterior+
        '" noInterior="'+cfdi.receptor.domicilio.noInterior+
        '" colonia="'+cfdi.receptor.domicilio.colonia+
        '" municipio="'+cfdi.receptor.domicilio.municipio+
        '" estado="'+cfdi.receptor.domicilio.estado+
        '" pais="'+cfdi.receptor.domicilio.pais+
        '" codigoPostal="'+cfdi.receptor.domicilio.codigoPostal+'" />');

        strList.Add('  </cfdi:Receptor>');

        strList.Add('  <cfdi:Conceptos>');
         for concepto in cfdi.conceptos do
          begin
            strList.Add('    <cfdi:Concepto cantidad="'+floattostr(concepto.cantidad)+
            '" unidad="'+concepto.unidad+
            '" noIdentificacion="'+concepto.noIdentificacion+
            '" descripcion="'+concepto.descripcion+
            '" valorUnitario="'+floattostr(concepto.valorUnitario)+
            '" importe="'+floattostr(concepto.importe)+ '" />');
          end;
        strList.Add('  </cfdi:Conceptos>');


        strList.Add('  <cfdi:Impuestos totalImpuestosTrasladados="'+floattostr(cfdi.impuestos.totalImpuestosTrasladados)+'">');
        strList.Add('    <cfdi:Traslados>');
        for traslado in cfdi.impuestos.traslados do
          begin
            strList.Add('      <cfdi:Traslado impuesto="'+GetEnumName(TypeInfo(TComprobanteImpuestosTrasladoImpuesto),integer(traslado.impuesto))+
            '" tasa="'+floattostr(traslado.tasa)+
            '" importe="'+floattostr(traslado.importe)+'" />');
          end;

        strList.Add('    </cfdi:Traslados>');
        strList.Add('  </cfdi:Impuestos>');
        strList.Add('</cfdi:Comprobante>');
        //strList.SaveToFile('C:\Users\User\Documents\Embarcadero\Studio\Projects\TimbraCFDI\GUIDemo\a.xml');

        idComprobante := cfdi.Emisor.rfc + cfdi.serie + cfdi.folio + inttostr(Random($270));
        if (Length(idComprobante)>$19) then idComprobante := leftstr(idComprobante,$19);

        try

            timbre := TResultadoTimbre.Create((X as TimbradoSoap).TimbraCFDI(_usuarioIntegrador,EncodeString(strList.Text),idComprobante));
            strList.Free;
        except on E: Exception do
          raise Exception.Create('Ocurri un error al timbrar: ' + E.Message);
        end;

        result:= timbre;

end;
   




end.
