unit ResultadoTimbre;

interface

uses Util;

type
   TResultadoTimbre = Class
     private
      _cadenaTimbre: string ;
      //_codigoBidimensional: array of byte;
      _descripcion: string ;
      _numeroError: string ;
      _tipoExcepcion: string ;
      _xml: string ;
      function _GetExitoso: Boolean;
      procedure ParseXML();

     // These methods and properties are all usable by instances of the class
     public
       codigoBidimensional: string; // TArray<Byte>; //array of byte;

       FolioUUID: string;
       No_Certificado: string;
       No_Certificado_SAT: string;
       FechaCertificacion: string;
       SelloCFDI: string;
       SelloSAT: string;
       //CadenaTimbre: string;
       
       constructor Create(resultados: ArrayOfAnyType);
       function GuardaCodigoBidimensional(ruta: string; nombreArchivo: string ): Boolean;
       function GuardaXML(ruta: string; nombreArchivo: string ): Boolean;
       
       property cadenaTimbre : String read _cadenaTimbre;
       //property codigoBidimensional : String read _codigoBidimensional;
       property descripcion : String read _descripcion;
       property numeroError : string read _numeroError;
       property tipoExcepcion : string read _tipoExcepcion;
       property xml : string read _xml;
       property Exitoso : Boolean read _GetExitoso;

   end;

implementation

uses
  System.SysUtils, StrUtils, System.Classes, Soap.EncdDecd,System.RegularExpressions;


constructor TResultadoTimbre.Create(resultados: ArrayOfAnyType);
 begin
    _tipoExcepcion := resultados[0];
    _numeroError := resultados[1];
    _descripcion := resultados[2];
    _xml := resultados[3];
    codigoBidimensional := resultados[4];//TEncoding.UTF8.GetBytes(resultados[4]);
    _cadenaTimbre := resultados[5];

    ParseXML();
    
 end;

function TResultadoTimbre._GetExitoso(): Boolean;
 begin

   Result := _numeroError='0';

 end;


procedure TResultadoTimbre.ParseXML();
var
   regexpr : TRegEx;
   match   : TMatch;
begin
   regexpr := TRegEx.Create('<cfdi:Complemento><tfd:TimbreFiscalDigital xmlns:tfd="\S+" version="\S+" UUID="(\S+)" FechaTimbrado="(\S+)" noCertificadoSAT="(\S+)" selloCFD="(\S+)=" selloSAT="(\S+)" xsi:schemaLocation=".+?" /></cfdi:Complemento>');
   match := regexpr.Match(_xml);
   if match.Success then
   begin
    FolioUUID:= match.Groups[1].Value;
    FechaCertificacion := match.Groups[2].Value;
    No_Certificado_SAT := match.Groups[3].Value;
    SelloCFDI := match.Groups[4].Value;
    SelloSAT := match.Groups[5].Value;
   end;
end;
 

function TResultadoTimbre.GuardaCodigoBidimensional(ruta: string; nombreArchivo: string ): Boolean;
 var
  cbStream: TBytesStream;
 begin

   if ( RightStr(lowercase(nombreArchivo),4)<> '.jpg') then
   begin
         nombreArchivo := nombreArchivo + '.jpg';
   end;


    try
         cbStream := TBytesStream.Create(DecodeBase64(codigoBidimensional));
         cbStream.SaveToFile(ruta + '' + nombreArchivo);
         cbStream.Free;
         result:=true;
    except on E: Exception do
      result:= false;
    end;


 end;




function TResultadoTimbre.GuardaXML(ruta, nombreArchivo: string): Boolean;
var
  MyText: TStringlist;
begin


      MyText:= TStringlist.create;
      try

        try
            MyText.Add(xml);
            MyText.SaveToFile(ruta + nombreArchivo, TEncoding.UTF8 );
        except on E: Exception do
           result:=false;
        end;
        
            result:=true;

      finally
        MyText.Free
      end;
  


end;

end.
